import vrml.*;
import vrml.node.*;
import vrml.field.*;

//
//
// ButtonTemplate
//
//
public class ButtonTemplate extends Script {

    public void initialize() {
        enabled              = (SFBool)getField("enabled");
        appearance           = (SFNode)getField("appearance");
        geometry             = (SFNode)getField("geometry");
        activeAppearance     = (SFNode)getField("activeAppearance");
        activeGeometry       = (SFNode)getField("activeGeometry");
        activeOverAppearance = (SFNode)getField("activeOverAppearance");
        activeOverGeometry   = (SFNode)getField("activeOverGeometry");
        overAppearance       = (SFNode)getField("overAppearance");
        overGeometry         = (SFNode)getField("overGeometry");
        disabledAppearance   = (SFNode)getField("disabledAppearance");
        disabledGeometry     = (SFNode)getField("disabledGeometry");
        downClicked          = (SFBool)getField("downClicked");
        upClicked            = (SFBool)getField("upClicked");
        pointerOver          = (SFBool)getField("pointerOver");
        active               = (SFBool)getField("active");

        appearance_changed = (SFNode)getEventOut("appearance_changed");
        geometry_changed   = (SFNode)getEventOut("geometry_changed");
        pressedTime        = (SFTime)getEventOut("pressedTime");

        browser = this.getBrowser();

        changeButton();

        System.out.println("ButtonTemplate loaded");
    }
    
    public void processEvent(Event event) {
        if (event.getName().equals("isOver")) {
            pointerOver.setValue((ConstSFBool)event.getValue());
            changeButton();
        } else {
            if (event.getName().equals("isActive")) {
                active.setValue((ConstSFBool)event.getValue());
                changeButton();
                if (active.getValue()) {
                    downClicked.setValue(true);
                } else {
                    if (downClicked.getValue()) {
                        upClicked.setValue(true);
                    }
                }
                
                if (upClicked.getValue()) {
                    if (downClicked.getValue() && pointerOver.getValue()) {
                        pressedTime.setValue(event.getTimeStamp());
                    }

                    // reset downClicked and upClicked
                    downClicked.setValue(false);
                    upClicked.setValue(false);
                }
            } else {
                if (event.getName().equals("set_enabled")) {
                    enabled.setValue((ConstSFBool)event.getValue());
                    enabled_changed.setValue(enabled.getValue());
                    changeButton();
                }
            }
        }
    }

    private void changeButton() {
        if (enabled.getValue()) {
            // the button is enabled
            if (pointerOver.getValue()) {
                // the pointer is over the geometry
                if (active.getValue()) {
                    appearance_changed.setValue(activeOverAppearance.getValue());
                    geometry_changed.setValue(activeOverGeometry.getValue());
                } else {
                    appearance_changed.setValue(overAppearance.getValue());
                    geometry_changed.setValue(overGeometry.getValue());
                }
            } else {
                if (active.getValue()) {
                    appearance_changed.setValue(activeAppearance.getValue());
                    geometry_changed.setValue(activeGeometry.getValue());
                } else {
                    // the pointer is not over the geometry
                    appearance_changed.setValue(appearance.getValue());
                    geometry_changed.setValue(geometry.getValue());
                }
            }
        } else {
            // the button is disabled
            appearance_changed.setValue(disabledAppearance.getValue());
            geometry_changed.setValue(disabledGeometry.getValue());
        }
    }

    private SFBool      enabled;
    private SFNode      appearance;
    private SFNode      geometry;
    private SFNode      activeAppearance;
    private SFNode      activeGeometry;
    private SFNode      activeOverAppearance;
    private SFNode      activeOverGeometry;
    private SFNode      overAppearance;
    private SFNode      overGeometry;
    private SFNode      disabledAppearance;
    private SFNode      disabledGeometry;
    private SFBool      downClicked;
    private SFBool      upClicked;
    private SFBool      pointerOver;
    private SFBool      active;

    private SFNode      appearance_changed;
    private SFNode      geometry_changed;
    private SFBool      enabled_changed;
    private SFTime      pressedTime;

    private Browser     browser;
}

